module Manoku
  class Application < Thor
    package_name 'Manoku'

    desc "new", "Create new Manoku project"
    def new(name)
      basedir = "#{workdir}/#{name.downcase.gsub(' ', '-')}"
      Dir.mkdir(basedir)
      init(basedir)
    end

    desc "init", "Initialize a Manoku configuration."
    def init(base=nil)
      base = workdir unless base
      Git.init(base)
      Dir.mkdir(appsdir)
      FileUtils.touch("#{appsdir}/.gitkeep")
      File.open("#{base}/.gitignore", "w") do |file|
        file << "apps/*.json"
      end
    end

    desc "add_key KEY", "Add Heroku API key."
    def add_key(key)
      @_heroku = nil
      FileUtils.rm(key_file) if File.exists?(key_file)
      File.open(key_file, "w") { |file| file << key }
    end

    desc "deploy", "Deploy a Manoku configuration."
    def deploy
      Dir["#{appsdir}/*.json"].each do |path|
        name = File.basename(path, '.json')
        json = JSON.parse(File.read(path))
        heroku.config_var.update(name, json)
        puts "Updated #{name} configuration."
      end
    end

    desc "download NAME", "Download the config variables for an app."
    def download(app)
      File.open("#{appsdir}/#{app}.json", "w") do |file|
        file << JSON.pretty_generate(heroku.config_var.info(app)).to_s
      end
    end

    private

    def workdir
      Dir.pwd
    end

    def appsdir
      "#{workdir}/apps"
    end

    def key_file
      "#{Dir.home}/.manoku"
    end

    def heroku
      @_heroku ||= PlatformAPI.connect_oauth(heroku_key)
    end

    def heroku_key
      File.read(key_file)
    end
  end
end
